package net.doubledoordev.mtrm.gui;

import net.doubledoordev.mtrm.MineTweakerRecipeMaker;
import net.doubledoordev.mtrm.network.MessageSend;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiTextField;
import net.minecraft.client.gui.inventory.GuiContainer;
import net.minecraft.client.resources.I18n;
import net.minecraft.inventory.ClickType;
import net.minecraft.inventory.Slot;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.client.config.GuiButtonExt;
import net.minecraftforge.fml.client.config.GuiCheckBox;
import net.minecraftforge.fml.client.config.GuiSlider;
import net.minecraftforge.fml.common.registry.GameData;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.oredict.OreDictionary;
import org.lwjgl.opengl.GL11;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static net.doubledoordev.mtrm.gui.MTRMContainer.RETURN_SLOT_ID;

/**
 * @author Dries007
 */
public class MTRMGui extends GuiContainer
{
    private static final ResourceLocation craftingTableGuiTextures = new ResourceLocation("mtrm:gui/mtrm.png");

    private static final Pattern MATCH_ALL = Pattern.compile("<\\*>");
    private static final Pattern META_WILDCARD = Pattern.compile(":\\*>");
    private static final Pattern ORE_DICT = Pattern.compile("<ore:.*>");
    private static final Pattern REUSE = Pattern.compile("\\.reuse\\(\\)");
    private static final Pattern NO_RETURN = Pattern.compile("\\.noReturn\\(\\)");
    private static final Pattern ANY_DAMAGE = Pattern.compile("\\.anyDamage\\(\\)");
    private static final Pattern ONLY_DAMAGE = Pattern.compile("\\.onlyDamaged\\(\\)");
    private static final Pattern ONLY_DAMAGE_AT_MOST = Pattern.compile("\\.onlyDamageAtMost\\((\\d+)\\)");
    private static final Pattern ONLY_DAMAGE_AT_LEAST = Pattern.compile("\\.onlyDamageAtLeast\\((\\d+)\\)");
    private static final Pattern ONLY_DAMAGE_BETWEEN = Pattern.compile("\\.onlyDamageBetween\\((\\d+), ?(\\d+)\\)");
    private static final Pattern WITH_DAMAGE = Pattern.compile("\\.withDamage\\((\\d+)\\)");
    private static final Pattern TRANSFORM_DAMAGE = Pattern.compile("\\.transformDamage\\((\\d+)\\)");
    private static final Pattern GIVE_BACK = Pattern.compile("\\.giveBack\\(<(.*):(.*)(:\\d+)?>(?: ?\\* ?(\\d+))\\)");
    private static final Pattern TRANSFORM_REPLACE = Pattern.compile("\\.transformReplace\\(<(.*):(.*)(:\\d+)?>(?: ?\\* ?(\\d+))\\)");

    private static final int ID_SEND = 10;
    private static final int ID_REMOVE = 11;
    private static final int ID_SHAPELESS = 12;
    private static final int ID_MIRRORED = 13;
    private static final int ID_OPTION_OK = 14;
    private static final int ID_OPTION_MATCHALL = 15;
    private static final int ID_OPTION_OREDICT = 16;
    private static final int ID_OPTION_META_WILDCARD = 17;
    private static final int ID_OPTION_NEXT_OREDICT = 18;
    private static final int ID_OPTION_ANY_DAMAGE = 19;
    private static final int ID_OPTION_WITH_DAMAGE = 20;
    private static final int ID_OPTION_ONLY_DAMAGED = 21;
    private static final int ID_CLOSE = 22;
    private static final int ID_OPTION_ONLY_DAMAGE_AT_LEAST = 23;
    private static final int ID_OPTION_ONLY_DAMAGE_AT_MOST = 24;
    private static final int ID_OPTION_ONLY_DAMAGE_BETWEEN = 25;
    private static final int ID_OPTION_REUSE = 26;
    private static final int ID_OPTION_ID_OPTION_TRANSFORM_DAMAGE = 27;
    private static final int ID_OPTION_NO_RETURN = 28;
    private static final int ID_OPTION_RETURN_OK = 29;
    private static final int ID_OPTION_ID_OPTION_TRANSFORM_REPLACE = 30;
    private static final int ID_OPTION_GIVE_BACK = 31;

    private final GuiSlider.ISlider iSlider = new GuiSlider.ISlider()
    {
        @Override
        public void onChangeSliderValue(GuiSlider slider)
        {
            tokenTxt.func_146180_a(getStackToken(false, field_147002_h.func_75139_a(editing).func_75211_c()));
        }
    };

    private final MTRMContainer container;
    private GuiCheckBox remove;
    private GuiCheckBox shapeless;
    private GuiCheckBox mirrored;
    private GuiCheckBox matchAll;
    private GuiCheckBox oreDict;
    private GuiCheckBox metaWildcard;
    private GuiCheckBox anyDamage;
    private GuiButtonExt optionsOk;
    private GuiButtonExt nextOreDict;
    private GuiTextField tokenTxt;
    private GuiCheckBox onlyDamaged;
    private GuiCheckBox withDamage;
    private GuiCheckBox onlyDamageAtLeast;
    private GuiCheckBox onlyDamageAtMost;
    private GuiCheckBox onlyDamageBetween;
    private GuiCheckBox reuse;
    private GuiCheckBox noReturn;
    private GuiCheckBox transformDamage;
    private GuiCheckBox transformReplace;
    private GuiCheckBox giveBack;
    private GuiButtonExt returnOk;
    private Map<GuiCheckBox, GuiSlider[]> sliders = new HashMap<>();
    private Map<GuiCheckBox, Pattern> patterns = new HashMap<>();
    private Map<GuiCheckBox, GuiCustomLabel[]> labels = new HashMap<>();

    private GuiCheckBox[] allDamage;
    private GuiCheckBox[] allTransform;
    private GuiCheckBox[][] allGroups;

    private MessageSend messageSend = new MessageSend();
    private int editing = -1;
    private int lastOreId = 0;
    private String errorMessage;

    public MTRMGui(MTRMContainer container)
    {
        super(container);
        field_147000_g = 181;
        this.container = container;
    }

    public String getStackToken(boolean nextOreDict, ItemStack stack)
    {
        boolean metaWildcard = this.metaWildcard.isChecked();
        boolean oreDict = this.oreDict.isChecked();
        if (stack == null) return "null";
        String stackName = GameData.getItemRegistry().func_177774_c(stack.func_77973_b()).toString();
        StringBuilder builder = new StringBuilder("<");
        if (oreDict)
        {
            int[] ids = OreDictionary.getOreIDs(stack);
            if (ids.length != 0)
            {
                stackName = "ore:" + OreDictionary.getOreName(ids[lastOreId]);
                if (nextOreDict) lastOreId++;
                if (lastOreId >= ids.length) lastOreId = 0;
            }
            else oreDict = false;
        }
        builder.append(stackName);
        if (!oreDict && (metaWildcard || stack.func_77952_i() != 0)) builder.append(':').append(metaWildcard || stack.func_77952_i() == OreDictionary.WILDCARD_VALUE ? "*" : stack.func_77952_i());
        builder.append('>');
        if (stack.field_77994_a > 1) builder.append(" * ").append(stack.field_77994_a);
        if (anyDamage.isChecked()) builder.append(".anyDamage()");
        if (onlyDamaged.isChecked()) builder.append(".onlyDamaged()");
        if (withDamage.isChecked()) builder.append(".withDamage(").append(sliders.get(withDamage)[0].getValueInt()).append(')');
        if (onlyDamageAtLeast.isChecked()) builder.append(".onlyDamageAtLeast(").append(sliders.get(onlyDamageAtLeast)[0].getValueInt()).append(')');
        if (onlyDamageAtMost.isChecked()) builder.append(".onlyDamageAtMost(").append(sliders.get(onlyDamageAtMost)[0].getValueInt()).append(')');
        if (onlyDamageBetween.isChecked()) builder.append(".onlyDamageBetween(").append(sliders.get(onlyDamageBetween)[0].getValueInt()).append(", ").append(sliders.get(onlyDamageBetween)[1].getValueInt()).append(')');
        if (reuse.isChecked()) builder.append(".reuse()");
        if (noReturn.isChecked()) builder.append(".noReturn()");
        if (transformDamage.isChecked()) builder.append(".transformDamage(").append(sliders.get(transformDamage)[0].getValueInt()).append(')');
        if (container.func_75139_a(RETURN_SLOT_ID).func_75216_d() && (giveBack.isChecked() || transformReplace.isChecked()))
        {
            ItemStack returnStack = container.func_75139_a(RETURN_SLOT_ID).func_75211_c();

            if (giveBack.isChecked()) builder.append(".giveBack(<");
            else if (transformReplace.isChecked()) builder.append(".transformReplace(<");

            builder.append(GameData.getItemRegistry().func_177774_c(returnStack.func_77973_b()));
            if (returnStack.func_77952_i() != 0) builder.append(':').append(returnStack.func_77952_i());
            builder.append('>');
            if (returnStack.field_77994_a > 1) builder.append(" * ").append(returnStack.field_77994_a);
            builder.append(')');
        }
        return builder.toString();
    }

    @Override
    public void func_73866_w_()
    {
        super.func_73866_w_();

        int id = 0;
        this.field_146292_n.add(new GuiButtonExt(id++, this.field_146294_l / 2 + 65, this.field_146295_m / 2 - 50, 10, 10, "*"));
        for (int y = 0; y < 3; ++y) for (int x = 0; x < 3; ++x) this.field_146292_n.add(new GuiButtonExt(id++, this.field_146294_l / 2 - 52 + x * 26, this.field_146295_m / 2 - 78 + y * 26, 10, 10, "*"));

        this.field_146292_n.add(remove = new GuiCheckBox(ID_REMOVE, this.field_146294_l / 2 + 90, this.field_146295_m / 2 - 80, "Remove", false));
        this.field_146292_n.add(shapeless = new GuiCheckBox(ID_SHAPELESS, this.field_146294_l / 2 + 90, this.field_146295_m / 2 - 70, "Shapeless", false));
        this.field_146292_n.add(mirrored = new GuiCheckBox(ID_MIRRORED, this.field_146294_l / 2 + 90, this.field_146295_m / 2 - 60, "Mirrored", true));
        this.field_146292_n.add(new GuiButtonExt(ID_SEND, this.field_146294_l / 2 + 90, this.field_146295_m / 2 - 45, 110, 20, "Ok!"));
        this.field_146292_n.add(new GuiButtonExt(ID_CLOSE, this.field_146294_l / 2 + 90, this.field_146295_m / 2 - 25, 110, 20, "Close"));

        int wOffset = this.field_146294_l / 2 - 200;
        int hOffset = this.field_146295_m / 2 - 110;

        tokenTxt = new GuiTextField(0, this.field_146289_q, wOffset, hOffset - 25, 220 + this.field_146999_f, 20);
        tokenTxt.func_146203_f(Integer.MAX_VALUE);

        this.field_146292_n.add(matchAll = new GuiCheckBox(ID_OPTION_MATCHALL, wOffset, hOffset += 10, "Match not empty (*)", false));
        this.field_146292_n.add(oreDict = new GuiCheckBox(ID_OPTION_OREDICT, wOffset, hOffset += 10, "Use ore dictionary", true));
        this.field_146292_n.add(metaWildcard = new GuiCheckBox(ID_OPTION_META_WILDCARD, wOffset, hOffset += 10, "Match any metadata", false));
        this.field_146292_n.add(anyDamage = new GuiCheckBox(ID_OPTION_ANY_DAMAGE, wOffset, hOffset += 10, "Any Damage", false));
        this.field_146292_n.add(onlyDamaged = new GuiCheckBox(ID_OPTION_ONLY_DAMAGED, wOffset, hOffset += 10, "Only Damaged", false));
        this.field_146292_n.add(withDamage = new GuiCheckBox(ID_OPTION_WITH_DAMAGE, wOffset, hOffset += 10, "With Damage X", false));
        this.field_146292_n.add(onlyDamageAtLeast = new GuiCheckBox(ID_OPTION_ONLY_DAMAGE_AT_LEAST, wOffset, hOffset += 10, "With Damage >= X", false));
        this.field_146292_n.add(onlyDamageAtMost = new GuiCheckBox(ID_OPTION_ONLY_DAMAGE_AT_MOST, wOffset, hOffset += 10, "With Damage < X", false));
        this.field_146292_n.add(onlyDamageBetween = new GuiCheckBox(ID_OPTION_ONLY_DAMAGE_BETWEEN, wOffset, hOffset += 10, "With X > Damage > Y", false));
        this.field_146292_n.add(transformDamage = new GuiCheckBox(ID_OPTION_ID_OPTION_TRANSFORM_DAMAGE, wOffset, hOffset += 10, "Transform Damage", false));
        this.field_146292_n.add(transformReplace = new GuiCheckBox(ID_OPTION_ID_OPTION_TRANSFORM_REPLACE, wOffset, hOffset += 10, "Transform Replace", false));
        this.field_146292_n.add(reuse = new GuiCheckBox(ID_OPTION_REUSE, wOffset, hOffset += 10, "Reuse", false));
        this.field_146292_n.add(noReturn = new GuiCheckBox(ID_OPTION_NO_RETURN, wOffset, hOffset += 10, "No Return", false));
        this.field_146292_n.add(giveBack = new GuiCheckBox(ID_OPTION_GIVE_BACK, wOffset, hOffset += 10, "GiveBack", false));

        this.field_146292_n.add(nextOreDict = new GuiButtonExt(ID_OPTION_NEXT_OREDICT, wOffset, hOffset += 40, 110, 20, "Next oredict value"));
        this.field_146292_n.add(optionsOk = new GuiButtonExt(ID_OPTION_OK, wOffset, hOffset += 20, 110, 20, "Save changes!"));

        // Buttongroups

        allDamage = new GuiCheckBox[]{matchAll, metaWildcard, anyDamage, onlyDamaged, withDamage, onlyDamageAtLeast, onlyDamageAtMost, onlyDamageBetween};
        allTransform = new GuiCheckBox[]{transformDamage, transformReplace, reuse, noReturn, giveBack};
        allGroups = new GuiCheckBox[][]{allDamage, allTransform};

        addPatterns(matchAll, MATCH_ALL);
        addPatterns(metaWildcard, META_WILDCARD);
        addPatterns(anyDamage, ANY_DAMAGE);
        addPatterns(onlyDamaged, ONLY_DAMAGE);

        addPatterns(reuse, REUSE);
        addPatterns(noReturn, NO_RETURN);

        id = 100;
        hOffset += 30;
        int hOffsetText = hOffset - this.field_147009_r - 10;

        GuiCustomLabel returnSlot = new GuiCustomLabel()
        {
            @Override
            public void draw(boolean b)
            {
                if (!b) func_73729_b(((field_146294_l - field_146999_f) / 2) - 110, ((field_146295_m - field_147000_g) / 2) + 142, 21, 20, 18, 18);
            }
        };
        addLabels(transformReplace, returnSlot, new GuiCustomLabelText("Transform Replace:", -110, hOffsetText - 68));
        addPatterns(transformReplace, TRANSFORM_REPLACE);
        returnSlot = new GuiCustomLabel()
        {
            @Override
            public void draw(boolean b)
            {
                if (!b) func_73729_b(((field_146294_l - field_146999_f) / 2) - 110, ((field_146295_m - field_147000_g) / 2) + 142, 21, 20, 18, 18);
            }
        };
        addLabels(giveBack, returnSlot, new GuiCustomLabelText("Give Back:", -110, hOffsetText - 68));
        addPatterns(giveBack, GIVE_BACK);
        this.field_146292_n.add(returnOk = new GuiButtonExt(ID_OPTION_RETURN_OK, wOffset + 20, hOffset - 68, 20, 18, "OK"));

        addSliders(withDamage, new GuiSlider(id++, wOffset, hOffset, 220 + this.field_146999_f, 20, "X = ", "", 0, 0, 0, false, true, iSlider));
        addLabels(withDamage, new GuiCustomLabelText("With Damage X", -110, hOffsetText));
        addPatterns(withDamage, WITH_DAMAGE);

        addSliders(onlyDamageAtLeast, new GuiSlider(id++, wOffset, hOffset, 220 + this.field_146999_f, 20, "X = ", "", 0, 0, 0, false, true, iSlider));
        addLabels(onlyDamageAtLeast, new GuiCustomLabelText("With Damage >= X", -110, hOffsetText));
        addPatterns(onlyDamageAtLeast, ONLY_DAMAGE_AT_LEAST);

        addSliders(onlyDamageAtMost, new GuiSlider(id++, wOffset, hOffset, 220 + this.field_146999_f, 20, "X = ", "", 0, 0, 0, false, true, iSlider));
        addLabels(onlyDamageAtMost, new GuiCustomLabelText("With Damage < X", -110, hOffsetText));
        addPatterns(onlyDamageAtMost, ONLY_DAMAGE_AT_MOST);

        addSliders(onlyDamageBetween, new GuiSlider(id++, wOffset, hOffset, 220 + this.field_146999_f, 20, "X = ", "", 0, 0, 0, false, true, iSlider), new GuiSlider(id++, wOffset, hOffset + 20, 220 + this.field_146999_f, 20, "Y = ", "", 0, 0, 0, false, true, iSlider));
        addLabels(onlyDamageBetween, new GuiCustomLabelText("With X > Damage > Y", -110, hOffsetText));
        addPatterns(onlyDamageBetween, ONLY_DAMAGE_BETWEEN);

        hOffset += 50;
        hOffsetText += 50;

        addSliders(transformDamage, new GuiSlider(id++, wOffset, hOffset, 220 + this.field_146999_f, 20, "X = ", "", 0, 0, 0, false, true, iSlider));
        addLabels(transformDamage, new GuiCustomLabelText("Transform with X damage", -110, hOffsetText));
        addPatterns(transformDamage, TRANSFORM_DAMAGE);

        setOptionsVisible(false);
    }

    private void addPatterns(GuiCheckBox key, Pattern values)
    {
        patterns.put(key, values);
    }

    private void addSliders(GuiCheckBox key, GuiSlider... values)
    {
        for (GuiSlider value : values) this.field_146292_n.add(value);
        sliders.put(key, values);
    }

    private void addLabels(GuiCheckBox key, GuiCustomLabel... values)
    {
        labels.put(key, values);
    }

    @Override
    protected void func_184098_a(Slot slot, int slotNumber, int mouseBtn, ClickType modifier)
    {
        super.func_184098_a(slot, slotNumber, mouseBtn, modifier);
        if (slot != null && slotNumber >= 0 && slotNumber <= 9)
        {
            showOptionsFor(slotNumber);
            messageSend.data[slotNumber] = getStackToken(false, slot.func_75211_c());
            showOptionsFor(slotNumber);
        }
        if (slotNumber == RETURN_SLOT_ID)
        {
            tokenTxt.func_146180_a(getStackToken(true, field_147002_h.func_75139_a(editing).func_75211_c()));
        }
    }

    protected void func_146284_a(GuiButton btn)
    {
        switch (btn.field_146127_k)
        {
            case ID_CLOSE:
                this.field_146297_k.field_71439_g.func_71053_j();
                break;
            case ID_SEND:
                messageSend.remove = remove.isChecked();
                messageSend.shapeless = shapeless.isChecked();
                messageSend.mirrored = mirrored.isChecked();
                MineTweakerRecipeMaker.getSnw().sendToServer(messageSend);
                break;
            case ID_REMOVE:
            case ID_SHAPELESS:
                mirrored.field_146124_l = !(shapeless.isChecked() || remove.isChecked());
                if (!mirrored.field_146124_l) mirrored.setIsChecked(false);
                break;
            case ID_OPTION_OK:
                saveOptions();
                break;
            case ID_OPTION_MATCHALL:
                radioBoxToggle(matchAll, allDamage);
                oreDict.setIsChecked(!matchAll.isChecked());
                oreDict.field_146124_l = !matchAll.isChecked();
                nextOreDict.field_146124_l = oreDict.isChecked();
                if (matchAll.isChecked()) tokenTxt.func_146180_a("<*>");
                else tokenTxt.func_146180_a(getStackToken(false, field_147002_h.func_75139_a(editing).func_75211_c()));
                break;
            case ID_OPTION_META_WILDCARD:
            case ID_OPTION_ONLY_DAMAGED:
            case ID_OPTION_ANY_DAMAGE:
            case ID_OPTION_WITH_DAMAGE:
            case ID_OPTION_ONLY_DAMAGE_AT_LEAST:
            case ID_OPTION_ONLY_DAMAGE_AT_MOST:
            case ID_OPTION_ONLY_DAMAGE_BETWEEN:
                radioBoxToggle((GuiCheckBox) btn, allDamage);
                tokenTxt.func_146180_a(getStackToken(false, field_147002_h.func_75139_a(editing).func_75211_c()));
                break;
            case ID_OPTION_ID_OPTION_TRANSFORM_DAMAGE:
            case ID_OPTION_ID_OPTION_TRANSFORM_REPLACE:
            case ID_OPTION_REUSE:
            case ID_OPTION_NO_RETURN:
            case ID_OPTION_GIVE_BACK:
                radioBoxToggle((GuiCheckBox) btn, allTransform);
                tokenTxt.func_146180_a(getStackToken(false, field_147002_h.func_75139_a(editing).func_75211_c()));
                break;
            case ID_OPTION_OREDICT:
                nextOreDict.field_146124_l = oreDict.isChecked();
            case ID_OPTION_NEXT_OREDICT:
            case ID_OPTION_RETURN_OK:
                tokenTxt.func_146180_a(getStackToken(true, field_147002_h.func_75139_a(editing).func_75211_c()));
                break;
            default:
                if (btn.field_146127_k >= 0 && btn.field_146127_k <= 10) showOptionsFor(btn.field_146127_k);
        }
    }

    private void radioBoxToggle(GuiCheckBox clicked, GuiCheckBox... others)
    {
        for (GuiCheckBox checkBox : others)
        {
            if (sliders.containsKey(checkBox)) for (GuiSlider s : sliders.get(checkBox)) s.field_146125_m = checkBox.isChecked();
            if (labels.containsKey(checkBox)) for (GuiCustomLabel s : labels.get(checkBox)) s.draw = checkBox.isChecked();
            if (checkBox == clicked) continue;
            if (clicked.isChecked()) checkBox.setIsChecked(false);
            checkBox.field_146124_l = !clicked.isChecked();
        }
    }

    private void saveOptions()
    {
        messageSend.data[editing] = tokenTxt.func_146179_b();
        if (messageSend.data[editing].equalsIgnoreCase("null")) messageSend.data[editing] = null;
        editing = -1;
        setOptionsVisible(false);
    }

    private void showOptionsFor(int id)
    {
        if (editing != id) lastOreId = 0;
        editing = id;
        String token = messageSend.data[id];
        if (token == null)
        {
            tokenTxt.func_146180_a("null");

            oreDict.field_146124_l = id != 0;
            oreDict.setIsChecked(id != 0);
            nextOreDict.field_146124_l = id != 0;

            for (GuiCheckBox[] group : allGroups)
            {
                for (GuiCheckBox checkBox : group)
                {
                    checkBox.setIsChecked(false);
                    if (sliders.containsKey(checkBox))
                    {
                        for (GuiSlider slider : sliders.get(checkBox))
                        {
                            slider.minValue = 0;
                            slider.maxValue = 0;
                            slider.setValue(0);
                        }
                    }
                    if (labels.containsKey(checkBox)) for (GuiCustomLabel l : labels.get(checkBox)) l.draw = false;
                }
            }
            container.func_75139_a(RETURN_SLOT_ID).func_75215_d(null);
        }
        else
        {
            tokenTxt.func_146180_a(token);

            oreDict.field_146124_l = id != 0;
            oreDict.setIsChecked(ORE_DICT.matcher(token).find() && id != 0);
            nextOreDict.field_146124_l = oreDict.isChecked() && id != 0;

            for (GuiCheckBox[] group : allGroups)
            {
                for (GuiCheckBox checkBox : group)
                {
                    if (patterns.containsKey(checkBox))
                    {
                        checkBox.setIsChecked(patterns.get(checkBox).matcher(token).find());
                        radioBoxToggle(checkBox, group);
                    }
                }
            }

            Matcher m = GIVE_BACK.matcher(token);
            if (m.find())
            {
                int meta = m.group(3) != null ? Integer.parseInt(m.group(3)) : 0;
                int size = m.group(4) != null ? Integer.parseInt(m.group(4)) : 1;
                Item i = Item.field_150901_e.func_82594_a(new ResourceLocation(m.group(1), m.group(2)));
                if (i != null) container.func_75139_a(RETURN_SLOT_ID).func_75215_d(new ItemStack(i, size, meta));
            }
        }

        for (GuiCheckBox[] group : allGroups)
        {
            for (GuiCheckBox checkBox : group)
            {
                checkBox.field_146124_l = id != 0;
                if (!sliders.containsKey(checkBox)) continue;
                GuiSlider[] sliderA = sliders.get(checkBox);
                for (GuiSlider slider : sliderA)
                {
                    slider.minValue = 0;
                    slider.maxValue = Integer.MAX_VALUE;
                    slider.setValue(0);
                    if (!oreDict.isChecked())
                    {
                        ItemStack stack = container.func_75139_a(editing).func_75211_c();
                        if (stack != null) slider.maxValue = stack.func_77973_b().func_77612_l();
                    }
                }
                if (token != null && patterns.containsKey(checkBox))
                {
                    Matcher matcher = patterns.get(checkBox).matcher(token);
                    if (matcher.find())
                    {
                        for (int i = 0; i < matcher.groupCount() && i < sliderA.length; i++)
                        {
                            sliderA[i].setValue(Integer.parseInt(matcher.group(i + 1)));
                        }
                    }
                }
            }
        }

        setOptionsVisible(true);
    }

    private void setOptionsVisible(boolean visible)
    {
        matchAll.field_146125_m = visible;
        metaWildcard.field_146125_m = visible;
        optionsOk.field_146125_m = visible;
        oreDict.field_146125_m = visible;
        nextOreDict.field_146125_m = visible;
        anyDamage.field_146125_m = visible;
        withDamage.field_146125_m = visible;
        onlyDamaged.field_146125_m = visible;
        reuse.field_146125_m = visible;
        returnOk.field_146125_m = visible;

        for (GuiCheckBox[] group : allGroups)
        {
            for (GuiCheckBox gui : group)
            {
                gui.field_146125_m = visible;
                if (sliders.containsKey(gui)) for (GuiSlider s : sliders.get(gui)) s.field_146125_m = visible && gui.isChecked();
                if (labels.containsKey(gui)) for (GuiCustomLabel l : labels.get(gui)) l.draw = visible && gui.isChecked();
            }
        }

        tokenTxt.func_146189_e(visible);

        if (!visible) container.func_75141_a(RETURN_SLOT_ID, null);
    }

//    protected void keyTyped(char p_73869_1_, int p_73869_2_)
//    {
//        if (!this.tokenTxt.textboxKeyTyped(p_73869_1_, p_73869_2_))
//        {
//            super.keyTyped(p_73869_1_, p_73869_2_);
//        }
//    }

//    protected void mouseClicked(int p_73864_1_, int p_73864_2_, int p_73864_3_)
//    {
//        super.mouseClicked(p_73864_1_, p_73864_2_, p_73864_3_);
//        this.tokenTxt.mouseClicked(p_73864_1_, p_73864_2_, p_73864_3_);
//    }

    /**
     * Draw the foreground layer for the GuiContainer (everything in front of the items)
     */
    protected void func_146979_b(int p_146979_1_, int p_146979_2_)
    {
        if (editing != -1)
        {
            this.field_146289_q.func_78276_b("Editing slot " + editing, -110, -55, 0xFFFFFF);
            this.field_146289_q.func_78276_b("Slot Options", -100, -20, 0xFFFFFF);
        }
        this.field_146289_q.func_78276_b("Recipe Options", this.field_146999_f + 15, 0, 0xFFFFFF);
        this.field_146289_q.func_78276_b("MineTweaker Recipe Maker", 28, 4, 4210752);
        this.field_146289_q.func_78276_b(I18n.func_135052_a("container.inventory"), 8, this.field_147000_g - 96 + 5, 4210752);
        if (errorMessage != null) this.func_73732_a(this.field_146289_q, errorMessage, this.field_146999_f / 2, -15, 0xFF0000);
        this.field_146289_q.func_78276_b("0", 144, 53, 0xFFFFFF);
        for (int y = 0; y < 3; ++y) for (int x = 0; x < 3; ++x) this.field_146289_q.func_78276_b(String.valueOf(1 + y * 3 + x), 28 + x * 26, 25 + y * 26, 0xFFFFFF);
        for (GuiCustomLabel[] labela : labels.values()) for (GuiCustomLabel label : labela) if (label.draw) label.draw(true);
    }

    protected void func_146976_a(float p_146976_1_, int p_146976_2_, int p_146976_3_)
    {
        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        this.field_146297_k.func_110434_K().func_110577_a(craftingTableGuiTextures);
        int k = (this.field_146294_l - this.field_146999_f) / 2;
        int l = (this.field_146295_m - this.field_147000_g) / 2;
        this.func_73729_b(k, l, 0, 0, this.field_146999_f, this.field_147000_g);
        for (GuiCustomLabel[] labela : labels.values()) for (GuiCustomLabel label : labela) if (label.draw) label.draw(false);
        tokenTxt.func_146194_f();
    }

    public void showMessage(String message)
    {
        this.errorMessage = message;
    }

    private abstract class GuiCustomLabel
    {
        boolean draw = false;

        public abstract void draw(boolean b);
    }

    private class GuiCustomLabelText extends GuiCustomLabel
    {
        String text;
        int color = 0xFFFFFF;
        int x, y;

        public GuiCustomLabelText(String s, int x, int y)
        {
            this.x = x;
            this.y = y;
            this.text = s;
        }

        public void draw(boolean b)
        {
            if (b) field_146289_q.func_78276_b(text, x, y, color);
        }
    }
}
